/* $Id: constants.hpp,v 1.44 2005/04/20 09:21:17 uwe Exp $ */

#ifndef  WOP_CONSTANTS_INCLUDED
#define  WOP_CONSTANTS_INCLUDED

/**********************************************************/
// some global string constants, that are used in several files

// name of the random circle theme
#define  RANDOM_CIRCLES_WOP_MAP_THEME  "randcircles"

// name of the file containing the theme definitions
// (only file name, no relative or absolute path)
#define  THEME_CONFIG_PATH  "images"
#define  THEME_CONFIG       "themes.conf"

// default relative path of the bonus configuration file
#define  THEME_DEFAULT_BONUS_CONFIG_PATH  "miscconf/bonus/standard.conf"

/**********************************************************
 * constants for game settings
 * according string maps in class WopSettings
 **********************************************************/

// program modes
enum {
	SERVER = 0,
	CLIENT,
	REPLAY,
	NUMBER_OF_PROGRAM_MODES
};

// view modes
enum {
	FULLSCREEN = 0,
	WINDOW,
	NUMBER_OF_VIEW_MODES
};

// game modes
enum {
	DEATHMATCH  = 0 << 0,
	TEAMPLAY    = 1 << 0,
	NUMBER_OF_GAME_MODES
};

const int MAX_NUMBER_OF_PLAYERS = 20;

/**********************************************************/

// Object-IDs
// Add new objects between NONE_VALID_OBJECT and NUMBER_OF_OBJECTS
// IMPORTANT: 
// (1) the valid IDs start with 0
// (2) the list MUST start with avatar IDs, terminated by the pseudo
//     ID LAST_AVATAR with value of the last avatar ID
// (3) also add the corresponding string in the char* array in
//     ObjectIDString in file constants.cpp
enum {
/// first, nonvalid ID 
	INVALID_OBJECT = -1,
/// avatar IDs
	FIRST_COLLIDABLE,
	FIRST_AVATAR = FIRST_COLLIDABLE,
		AVATAR_WORM = FIRST_AVATAR,
	// (1) insert new avatar IDs above this line
	// (2) set LAST_AVATAR to the new avatar ID
	LAST_AVATAR = AVATAR_WORM,
/// IDs of other collidable objects
	//
	BALL,
	GRENADE,
	MINE,
	MISSILE,
	HOMING_MISSILE,
	HOOK,
	// bonus objects
	FIRST_BONUS,
		BONUS_HEALTH = FIRST_BONUS,
		BONUS_SHIELD,
		BONUS_INVISIBLE,
		BONUS_FUEL,
		BONUS_HOMING_VIRUS,
	LAST_BONUS = BONUS_HOMING_VIRUS,
	// 
	LAST_COLLIDABLE = LAST_BONUS,
/// IDs of noncollidable objects
	FIRST_NONCOLLIDABLE,
		// IDs of explosions. We need SMALLEST_EXPLOSION
		// and BIGGEST_EXPLOSION to check the range of
		// explosion sizes in the program.
		SMALLEST_EXPLOSION = FIRST_NONCOLLIDABLE,
			// first (== smallest) Explosion must be SMALLEST_EXPLOSION
			EXPLOSION_15 = SMALLEST_EXPLOSION,
			EXPLOSION_40,
			EXPLOSION_60,
			EXPLOSION_80,
		BIGGEST_EXPLOSION = EXPLOSION_80,
		// IDs of smoke. We need SMALLEST_SMOKE and
		// BIGGEST_SMOKE to check the range of smoke sizes
		// in the program.
		SMALLEST_SMOKE,
			SMOKE_15 = SMALLEST_SMOKE,
			SMOKE_30,
		BIGGEST_SMOKE = SMOKE_30,
		// IDs of blue glowing effects. We need
		// SMALLEST_BLUEGLOW and BIGGEST_BLUEGLOW to check the
		// range of effect sizes in the program.
		SMALLEST_BLUEGLOW,
			BLUEGLOW_25 = SMALLEST_BLUEGLOW,
		BIGGEST_BLUEGLOW = BLUEGLOW_25,
		// goals of different size
		SMALLEST_GOAL,
			GOAL_50 = SMALLEST_GOAL,
		BIGGEST_GOAL = GOAL_50,
		// IDs of shield corona
		SHIELD_CORONA,
		// IDs of invisible corona
		INVISIBLE_CORONA,
		ROPE,
	// (1) insert new noncollidabel object above this line
	// (2) adapt LAST_NONCOLLIDABLE to this new ID
	LAST_NONCOLLIDABLE = ROPE,
/// the total number of valid IDs
	NUMBER_OF_OBJECT_IDs
};

// extern string IDs
extern const char* const ObjectIDString[NUMBER_OF_OBJECT_IDs+1];

/**********************************************************/

enum {
	WEAPON_SHOTGUN = 0,
	WEAPON_UZI,
	WEAPON_RUBBERGUN,
	WEAPON_GRENADE,
	WEAPON_MINE,
	WEAPON_MISSILE,
	WEAPON_HOMING_MISSILE,
	WEAPON_ZAPPER,
	WEAPON_HOOK,
	NUMBER_OF_WEAPON_IDs
};

extern const char* const WeaponIDString[NUMBER_OF_WEAPON_IDs+1];

/**********************************************************/

#endif // WOP_CONSTANTS_INCLUDED
