/* $Id: explosion.cpp,v 1.9 2005/01/13 17:29:54 uwe Exp $ */

#include  "explosion.hpp"
#include  "global.hpp"
#include  "world.hpp"
#include  "serialize.hpp"

/**********************************************************/

Explosion::Explosion( const Sint32 size )
         : m_size( SMALLEST_EXPLOSION )
{
	// sequence is chosen randomly in setSize()
	if( m_worldPointer->getSpriteInterface() ) setSize( size );
}

/**********************************************************/

inline void Explosion::setSize( const Sint32 size )
{
	DBG( 1 ) ASSERT( getMinSize() <= size && size <= getMaxSize(),
	                 "Explosion::setSize: size %d is out of range [%d,%d]\n",
	                 size, getMinSize(), getMaxSize() );

	// set new size
	m_size = size;

////////////////////////////////////////////////
//// commented out this check on 27/06/2004 ////
//// I think, we do not need it ////////////////
//	if(      m_size < getMinSize() )  m_size = getMinSize();
//	else if( m_size > getMaxSize() )  m_size = getMaxSize();
////////////////////////////////////////////////

	// choose a sequence randomly
	// this also sets m_SequenceLength
	setSequence( globalRnd.getUint32() %
	             m_worldPointer->getSpriteInterface()->getNumSequences(*this) );
	// start sequence in any case
	m_Frame =  0;
}

/**********************************************************/

void Explosion::update()
{
	// explosions only live for one cycle
	if( ++m_Frame >= m_SequenceLength ) {
		m_worldPointer->deleteObject( this );
		// DO NOT TOUCH OBJECT AFTER deleteObject
		return;
	}
}
/******************************************************************************/

void Explosion::serialize( Uint8*& bufferPointer ) const {
	// expands to a check of the buffer movement
	START_OBJECT_SERIALIZED_SIZE_CHECK( bufferPointer );

	NonCollidableObject::serialize( bufferPointer );	

	Serialize<Sint32>::serialize( m_size, bufferPointer );

	// expands to tag serialization
	SERIALIZE_OBJECT_TAG( bufferPointer );
	// expands to a check of the buffer movement
	END_OBJECT_SERIALIZED_SIZE_CHECK( bufferPointer, Explosion );
}

/******************************************************************************/

void Explosion::deserialize( Uint8*& bufferPointer ) {
	NonCollidableObject::deserialize( bufferPointer );

	Serialize<Sint32>::deserialize( bufferPointer, m_size );

	// expands to tag deserialization
	DESERIALIZE_OBJECT_TAG( bufferPointer );
}

/******************************************************************************/

Uint32 Explosion::getSerializeBufferSize() const {
	return   NonCollidableObject::getSerializeBufferSize() 
	       + Serialize<Sint32>::sizeOf( m_size )
	         // adds additional size for debugging (see serialize.hpp),
	         // but only, if the tags are used
	         PLUS_TAG_SIZE( 1 );
}

/******************************************************************************/
