/* $Id: homingmissile.cpp,v 1.12 2005/04/21 14:50:05 pohlt Exp $ */

#include "homingmissile.hpp"
#include "avatar.hpp"
#include "player.hpp"
#include "world.hpp"
#include "smoke.hpp"

/**********************************************************/

#define  MASS                    50.0

#define  ACCELERATE_FUEL         10
#define  MIN_ACCELERATION         5
#define  CHASE_FUEL             100
#define  CHASING_THRUST_IMPACT    1.0
#define  RANGE_OF_SIGHT         220.0
#define  SPEED_PREDICTION         3.0
// dependent constants
#define  TOTAL_FUEL             (ACCELERATE_FUEL + CHASE_FUEL)
#define  RANGE_OF_SIGHT_SQUARE  (RANGE_OF_SIGHT * RANGE_OF_SIGHT)

/**********************************************************/

HomingMissile::HomingMissile()
{
//
// take the values from class Missile
//
//	m_health = 300000;
	m_mass   = MASS;
//	
//	m_collRectX = -4;
//	m_collRectY = -4;
//
//	m_collRectWidth  = 9;
//	m_collRectHeight = 9;
//
}

/**********************************************************/

bool HomingMissile::initialize( const Vector& pos,
                                const Vector& vel,
                                const Uint8   owner,
                                const Sint32  nShrapnels,
                                const Sint32  damage,
                                const Vector& thrust )
{
	return
	Missile::initialize( pos, vel, owner, nShrapnels,
	                     damage, TOTAL_FUEL, thrust );
}

/**********************************************************/

void HomingMissile::update()
{
	if( m_damageSampleTimeout > 0 ) m_damageSampleTimeout--;

	Map*  const map = m_worldPointer->getMap();
	const real DT  = 1.0;
	int cx, cy, pcx, pcy;

	// remove missile stencil
	removeCollRect();

	// check for prey
	const std::vector<Player*>& players = m_worldPointer->getPlayerList();
	const Avatar* prey = NULL;
	real minDistSquare = RANGE_OF_SIGHT_SQUARE;
	// only check for prey, if we could chase it
	if( m_accelTimer > 0 ) {
		for( Uint8 p = 0; p < players.size(); p++ ) {
			// do not chase the missile's owner
			if( m_owner != players[p]->getPlayerID() ) {
				const Avatar* avatar   = players[p]->getAvatar();
				if( avatar->isLiving() && avatar->isWeaponVisible() ) {
					const real distX      = avatar->getPositionX()
					                          - getPositionX(),
					            distY      = avatar->getPositionY()
					                          - getPositionY(),
					            distsquare = (distX * distX) + (distY * distY);
					// search the nearest avatar
					// NOTE that due to the initialization of minDistSquare
					// distsquare will also be < RANGE_OF_SIGHT_SQUARE.
					if( distsquare < minDistSquare ) {
						minDistSquare = distsquare;
						prey          = avatar;
					}
				}
			}
		}
	}

	bool ignition = false;
	// we found a prey
	if( prey != NULL ) {
		// if we still have fuel AND
		if( m_accelTimer > 0 ) {
			// we are not in the initial start phase
			if( m_accelTimer < (TOTAL_FUEL - MIN_ACCELERATION) ) {
				// turn thrust
				m_thrust = prey->getPosition() + prey->getSpeed() * SPEED_PREDICTION -
				                 getPosition() -       getSpeed() * SPEED_PREDICTION;
				m_thrust.normalize() *= CHASING_THRUST_IMPACT;
				// ignition towards the prey
				ignition = true;
			} else {
				// ignition in the initial start phase
				ignition = true;
				prey     = NULL;
			}
		}
	// we did not find a prey
	} else {
		// we are still in the neutral acceleration phase
		if( m_accelTimer > CHASE_FUEL ) {
			ignition = true;
		}
	}

	// engine is running
	if( ignition ) {
		// accelerate in direction of thrust
		m_speedX += m_thrust.x;
		m_speedY += m_thrust.y;
		// throw out some booster sparks
		for( int i = 0; i < 15; i++ ) {
			m_worldPointer->getParticles()->addParticle( Particles::ParticleData(
			     getPosition(),
			     getSpeed() - m_thrust*10.0 + globalRnd.getVector( 1.5 ),
			     Particles::BOUNCE_EARTH | //Particles::BOUNCE_OBJECT |
			     Particles::REDUNDANT | Particles::SPARK, 2, 0,
			     Particles::m_N_SPARK_COLORS/2 + globalRnd.getUint32() % (Particles::m_N_SPARK_COLORS/2+1), 1,
			     0xffffff));
		}
		// generate a smoke cloud
		Smoke* cloud = static_cast<Smoke*>(m_worldPointer->newObject(SMOKE_30));
		cloud->setPosition( getPosition() + localRnd.getVector( 4.0 ));
		cloud->setSpeed( getSpeed() + localRnd.getVector( 2.0 ));
		// color the cloud, if we chase someone
		if( prey != NULL ) {
			cloud->setColor( prey->getPlayerColor() );
			// lock the avatar (maybe it has got some authomatic defense)
			((Avatar*)prey)->getLockedByHomingMissile( this, minDistSquare );
		}
		// decrement "fuel"
		if( m_accelTimer > 0 )  m_accelTimer--;
	}
	// move missile
	m_speedY += GRAVITATION * DT;

	// calcuate new position
	real newPosX = m_positionX + m_speedX * DT,
	      newPosY = m_positionY + m_speedY * DT;

	// is there something to collide?
	if( map->testLineCollRect( ROUND( m_positionX ) + m_collRectX,
	                           ROUND( m_positionY ) + m_collRectY,
	                           ROUND( newPosX     ) + m_collRectX,
	                           ROUND( newPosY     ) + m_collRectY,
	                           m_collRectWidth, m_collRectHeight,
	                           pcx, pcy, cx, cy ) ) {
		// no, then just keep on flying
		m_positionX = newPosX;
		m_positionY = newPosY;
	} else {
		m_positionX = pcx - m_collRectX;
		m_positionY = pcy - m_collRectY;
		if( ! m_firstUpdate )  m_health = 0; // explode if not first update
	}

	// explode?
	if( m_health <= 0 ) {
		Audio::getInstance()->playSound( m_explodeSample, ROUND( m_positionX ),
		                                                  ROUND( m_positionY ) );
		m_worldPointer->newObject( EXPLOSION_80 )->setPosition( ROUND( m_positionX ),
		                                                        ROUND( m_positionY ));
		for( int s = 0; s < m_nShrapnels; s++ ) {
			m_worldPointer->getParticles()->addParticle( Particles::ParticleData( getPosition(),
			     globalRnd.getVector( 15 + 5*globalRnd.getNormedReal() ), // + getSpeed(),  otherwise shrapnels will just fly forward
			     Particles::EXPL_EARTH | Particles::EXPL_OBJECT | Particles::DAMAGE |
			     Particles::INVISIBLE,
			     0, m_owner, 2, m_damage, 0xffffff ));
		}

		// remove missile object
		m_worldPointer->deleteObject( this );
		// DO NOT TOUCH ANY MISSILE DATA MEMBERS AFTER THIS POINT
		return;
	}

	// increment the animation frame
	if( ++m_Frame >= m_SequenceLength ) m_Frame = 0;
	// select new missile sprite
	updateSpriteSelection( ignition );

	// set missile stencil
	setCollRect();

	m_firstUpdate = false;

	DBG( 5 ) map->setColorAt( ROUND( m_positionX ),
	                          ROUND( m_positionY ), 0xffffff );
}

/**********************************************************/

void HomingMissile::updateSpriteSelection( const bool activeEngine )
{
	// If the engine is running, face in the negative direction
	// of the thrust vector, otherwise in direction of movement.
	const real angle =   activeEngine
	                    ? ATAN2_REAL( m_thrust.y, m_thrust.x )
	                    : ATAN2_REAL( m_speedY,   m_speedX   );
	// set the sequence index.
	// NOTE: setSequence also updates m_SequenceLength.
	setSequence( (ROUND( angle*(18.0/M_PI) - 0.5 )+36) % 36 );

	// only reset the frame index, if we cannot continue
	// this movement in a different aiming angle
	if( getFrame() >= m_SequenceLength )  setFrame( 0 );
}

/**********************************************************/
