/* $Id: object.cpp,v 1.26 2005/04/08 13:16:03 pohlt Exp $ */

#include "object.hpp"
#include "world.hpp"
#include "serialize.hpp"

/**********************************************************/

World* Object::m_worldPointer = NULL;

/**********************************************************/

Object::Object()
      : m_positionX ( 0 ),
        m_positionY ( 0 ),
        m_speedX ( 0 ),
        m_speedY ( 0 ),
        m_state( VISIBLE | WEAPON_VISIBLE ),
        m_Sequence( 0 ),
        m_SequenceLength( 1 ),
        m_Frame( 0 ),
        m_ColorInstance( -1 ) {
	setState( VISIBLE, true );
	setState( GROUNDED, false );
}

/**********************************************************/

Object::~Object() {
	LOG( 5 ) INFO( "Object::~Object: done\n" );
}

/**********************************************************/

void Object::updateSequenceLength()
{
	DBG( 1 ) ASSERT( m_worldPointer, "Object::updateSequenceLength: "
	                 "m_worldPointer == NULL\n" );

	if( m_worldPointer->getSpriteInterface() ) {
		m_SequenceLength = m_worldPointer
			->getSpriteInterface()
				->getSequenceLength(this->getID(), m_Sequence);
	}
}

/**********************************************************/

void Object::serialize( Uint8*& bufferPointer ) const
{
	// expands to a check of the buffer movement
	START_OBJECT_SERIALIZED_SIZE_CHECK( bufferPointer );
	// expands to tag serialization
	SERIALIZE_OBJECT_TAG( bufferPointer );
	
	Serialize<real>::serialize( m_positionX, bufferPointer );
	Serialize<real>::serialize( m_positionY, bufferPointer );
	Serialize<real>::serialize( m_speedX, bufferPointer );
	Serialize<real>::serialize( m_speedY, bufferPointer );
	Serialize<Uint32>::serialize( m_state, bufferPointer );
	Serialize<Sint32>::serialize( m_Sequence, bufferPointer );
	Serialize<Sint32>::serialize( m_SequenceLength, bufferPointer );
	Serialize<Sint32>::serialize( m_Frame, bufferPointer );
	// do not serialize the index for the colored instance, since
	// the instance must be created independently on each client

	// expands to tag serialization
	SERIALIZE_OBJECT_TAG( bufferPointer );
	// expands to a check of the buffer movement
	END_OBJECT_SERIALIZED_SIZE_CHECK( bufferPointer, Object );
}

/**********************************************************/

void Object::deserialize( Uint8*& bufferPointer )
{
	// expands to tag deserialization
	DESERIALIZE_OBJECT_TAG( bufferPointer );
	
	Serialize<real>::deserialize( bufferPointer, m_positionX );
	Serialize<real>::deserialize( bufferPointer, m_positionY );
	Serialize<real>::deserialize( bufferPointer, m_speedX );
	Serialize<real>::deserialize( bufferPointer, m_speedY );
	Serialize<Uint32>::deserialize( bufferPointer, m_state );
	Serialize<Sint32>::deserialize( bufferPointer, m_Sequence );
	Serialize<Sint32>::deserialize( bufferPointer, m_SequenceLength );
	Serialize<Sint32>::deserialize( bufferPointer, m_Frame );
	// do not deserialize the index for the colored instance, since
	// the instance must be created independently on each client

	// expands to tag deserialization
	DESERIALIZE_OBJECT_TAG( bufferPointer );
}

/**********************************************************/

Uint32 Object::getSerializeBufferSize() const
{
	return   4 * Serialize<real>::sizeOf()
	       +     Serialize<Uint32>::sizeOf( m_state )
	         // NOTE: m_ColoredInstance is not (de)serialized
	       + 3 * Serialize<Sint32>::sizeOf()
	         // adds additional size for debugging (see serialize.hpp),
	         // but only, if the tags are used
	         PLUS_TAG_SIZE( 2 );
}

/**********************************************************/
