/* $Id: wopgui.cpp,v 1.28 2005/05/13 09:22:27 pohlt Exp $ */

#include "wopgui.hpp"

#include <string>
#include <sstream>

#include "client.hpp"
#include "string.hpp"
#include "wopsettings.hpp"

#include "fontfactory.h"
#include "layout.h"
#include "image.h"

using namespace SDLwidgets;

WopGUI::WopGUI( Client* client, SDL_Surface* surface )
	: m_client       ( client ),
	  m_splashScreen ( 0 ),
	  m_surfaceWidth ( surface->w ),
	  m_surfaceHeight( surface->h ) {
	const SDL_Color black     = {  0,   0,   0, 0},
	                lightBlue = {100, 100, 255, 0};
	Widget::setDefaultColor( black );
	TextButton::setDefaultColor( black );
	TextButton::setDefaultHighlightColor( lightBlue );
	TextButton::setDefaultHighlightAlpha( 120 );
	
	m_mutex = SDL_CreateMutex();
	
	std::string fontPath( WopSettings::getInstance()->getData() );
	fontPath += "/images/gui";
	SDLwidgets::FontFactory::getInstance()->setFontPath( fontPath );
	
	m_root = new RootContainer( surface );
	
	const SDL_Color background = {0, 0, 255, 0};
	m_chatInputWindow.input = new TextField( 600, 20 );
	m_chatInputWindow.input->setFont( "tahoma.ttf", 12 );
	m_chatInputWindow.input->addActionListener( this );
	m_chatInputWindow.window = new WopWindow( m_chatInputWindow.input, "Enter chat message" );
	m_chatInputWindow.window->setPosition( 16, 150 );
	
	m_messageWindow.messageArea = new MessageWidget( 560, 50 );
	m_messageWindow.messageArea->setFont( "tahomabd.ttf", 11 );
	m_messageWindow.window = new BackgroundWidget( m_messageWindow.messageArea, background, 20 );
	m_messageWindow.window->setPosition( 0, 430 );
	
	m_weaponWindow.indicators = new WeaponWindow();
	m_weaponWindow.window = new BackgroundWidget( m_weaponWindow.indicators, background, 20 );
	m_weaponWindow.window->setWidth( 75 );
	m_weaponWindow.window->setPosition( 560, 430 );
	
	std::ostringstream path;
	path << WopSettings::getInstance()->getData();
	path << "/images/gui/health.png";
	Image* healthImage = new Image( path.str().c_str() );
	m_healthWidget.indicator = new WeaponIndicator( 75, 12 );
	const SDL_Color healthColor = {207, 0, 0, 0};
	m_healthWidget.indicator->setColor( healthColor );
	m_healthWidget.container = new WidgetComposite( 0, 0 );
	HorizontalLayout* healthLayout = new HorizontalLayout( m_healthWidget.container );
	m_healthWidget.container->setLayout( healthLayout );
	m_healthWidget.container->add( healthImage );
	m_healthWidget.container->add( m_healthWidget.indicator );
	m_healthWidget.container->setPosition( 551, 5 );
	
	std::ostringstream path2;
	path2 << WopSettings::getInstance()->getData();
	path2 << "/images/gui/rope.png";
	Image* ropeImage = new Image( path2.str().c_str() );
	m_ropeWidget.indicator = new WeaponIndicator( 75, 12 );
	const SDL_Color ropeColor = {219, 205, 0, 0};
	m_ropeWidget.indicator->setColor( ropeColor );
	m_ropeWidget.container = new WidgetComposite( 0, 0 );
	HorizontalLayout* ropeLayout = new HorizontalLayout( m_ropeWidget.container );
	m_ropeWidget.container->setLayout( ropeLayout );
	m_ropeWidget.container->add( ropeImage );
	m_ropeWidget.container->add( m_ropeWidget.indicator );
	m_ropeWidget.container->setPosition( 551, 7 + m_healthWidget.container->getHeight() );

	std::ostringstream path3;
	path3 << WopSettings::getInstance()->getData();
	path3 << "/images/gui/ammo.png";
	Image* activeWeaponImage = new Image( path3.str().c_str() );
	m_activeWeaponWidget.indicator = new WeaponIndicator( 75, 12 );
	const SDL_Color ammoColor = {167, 167, 217, 0};
	m_activeWeaponWidget.indicator->setColor( ammoColor );
	m_activeWeaponWidget.container = new WidgetComposite( 0, 0 );
	HorizontalLayout* activeWeaponLayout = new HorizontalLayout( m_activeWeaponWidget.container );
	m_activeWeaponWidget.container->setLayout( activeWeaponLayout );
	m_activeWeaponWidget.container->add( activeWeaponImage );
	m_activeWeaponWidget.container->add( m_activeWeaponWidget.indicator );
	m_activeWeaponWidget.container->setPosition( 551, 9 + m_healthWidget.container->getHeight() + m_ropeWidget.container->getHeight() );

	std::ostringstream path4;
	path4 << WopSettings::getInstance()->getData();
	path4 << "/images/gui/fuel.png";
	Image* fuelImage = new Image( path4.str().c_str() );
	m_fuelWidget.indicator = new WeaponIndicator( 75, 12 );
	const SDL_Color fuelColor = {66, 181, 0, 0};
	m_fuelWidget.indicator->setColor( fuelColor );
	m_fuelWidget.container = new WidgetComposite( 0, 0 );
	HorizontalLayout* fuelLayout = new HorizontalLayout( m_fuelWidget.container );
	m_fuelWidget.container->setLayout( fuelLayout );
	m_fuelWidget.container->add( fuelImage );
	m_fuelWidget.container->add( m_fuelWidget.indicator );
	m_fuelWidget.container->setPosition( 551, 11 + m_healthWidget.container->getHeight() + m_ropeWidget.container->getHeight()+m_activeWeaponWidget.container->getHeight() );

	m_serverListWindow = 0;
	
	m_scoreTableWindow.scoreTable = new ScoreTable();
	m_scoreTableWindow.window = new WopWindow( m_scoreTableWindow.scoreTable, "Scores" );
	m_scoreTableWindow.window->setPosition( 100, 70 );
	
	m_initializationWindow.progressBar = new WeaponIndicator( 200, 20 );
	const SDL_Color progressBarColor = {50, 50, 255, 0};
	m_initializationWindow.progressBar->setColor( progressBarColor );
	m_initializationWindow.title = new TextLabel( "tahoma.ttf", 12, "Stand by..." );
	m_initializationWindow.title->setColor( black );
	m_initializationWindow.title->setPosition( 0, 21 );
	WidgetComposite* initializationWidget = new WidgetComposite();
	initializationWidget->setLayout( new VerticalLayout( initializationWidget ) );
	initializationWidget->add( m_initializationWindow.progressBar );
	initializationWidget->add( m_initializationWindow.title );
	m_initializationWindow.window = new WopWindow( initializationWidget, "Loading sprites" );
	m_initializationWindow.window->setPosition( 210, 360 );
	  
	m_optionsWindow = new OptionsWindow( WopSettings::getInstance(), 540, 400 );
	m_optionsWindow->addActionListener( this );
	m_optionsWindow->setPosition( ( m_surfaceWidth - m_optionsWindow->getWidth() ) / 2,
	                              ( m_surfaceHeight - m_optionsWindow->getHeight() ) / 2 );
	
	createCreditsWindow();
	createMainMenu();
}

WopGUI::~WopGUI() {
	delete m_chatInputWindow.window;
	delete m_messageWindow.window;
	delete m_weaponWindow.window;
	SDL_DestroyMutex( m_mutex );
}

void WopGUI::actionPerformed( Uint32 cmd, void* source ) {
	if ( source == m_mainMenu.startButton && cmd == Button::BUTTON_PRESSED ) {
		hideMainMenu();
		SDL_mutexP( m_mutex );
		std::set<ServerEntry> entries = m_client->lookForServer();
		SDL_mutexV( m_mutex );
		showServerListWindow( entries );
	}
	else if ( source == m_mainMenu.optionsButton && cmd == Button::BUTTON_PRESSED ) {
		hideMainMenu();
		showOptionsWindow();
	}
	else if ( source == m_mainMenu.creditButton && cmd == Button::BUTTON_PRESSED ) {
		hideMainMenu();
		showCreditsWindow();
	}
	else if ( source == m_mainMenu.exitButton && cmd == Button::BUTTON_PRESSED ) {
		m_client->exit();
	}
	else if ( source == m_creditsWindow.backButton && cmd == Button::BUTTON_PRESSED ) {
		hideCreditsWindow();
		showMainMenu();
	}
	else if ( source == m_optionsWindow && ( cmd == OptionsWindow::BACK_PRESSED ||
	                                         cmd == OptionsWindow::SAVE_PRESSED ) ) {
		hideOptionsWindow();
		showMainMenu();
	}
	else if ( source == m_serverListWindow ) {
		switch( cmd ) { // FIXME: move variable declarations out of cases
		case ServerListWindow::BACK_PRESSED:
			hideServerListWindow();
			showMainMenu();
			break;
		case ServerListWindow::REFRESH_PRESSED: {
			hideServerListWindow();
			SDL_mutexP( m_mutex );
			m_serverListWindow->clearEntries();
			std::set<ServerEntry> entries = m_client->lookForServer();
			SDL_mutexV( m_mutex );
			showServerListWindow( entries );
			break;
		}
		case ServerListWindow::CONNECT_PRESSED:
			if ( m_serverListWindow->getSelectedIndex() != -1 ) {
				hideServerListWindow();
				m_client->connectToServer( m_serverListWindow->getSelectedServer() );
			}
			break;
		}
	}
	else if ( source == m_chatInputWindow.input ) {
		switch ( cmd ) { // FIXME: move variable declarations out of cases
		case TextField::EDITING_FINISHED:
			if ( m_chatInputWindow.input->getText().size() > 0 ) {
				String message( m_chatInputWindow.input->getText().c_str() );
				m_client->sendChatMessage( message );
				m_chatInputWindow.input->clear();
			}
			SDL_mutexP( m_mutex );
			m_root->remove( m_chatInputWindow.window );
			m_root->removeFocus( m_chatInputWindow.input );
			SDL_mutexV( m_mutex );
			break;
		case TextField::EDITING_CANCELLED:
			SDL_mutexP( m_mutex );
			m_root->remove( m_chatInputWindow.window );
			m_root->removeFocus( m_chatInputWindow.input );
			SDL_mutexV( m_mutex );
			break;
		}
	}
}

void WopGUI::showMessage( String message, const SDL_Color& color ) {
	SDL_mutexP( m_mutex );
	std::string s( message.getString() );
	m_messageWindow.messageArea->addMessage( s, color );
	SDL_mutexV( m_mutex );
}

void WopGUI::showServerListWindow( std::set<ServerEntry>& servers ) {
	SDL_mutexP( m_mutex );
	m_serverListWindow = new ServerListWindow( 540, 400 );
	m_serverListWindow->setEntries( servers );
	m_serverListWindow->addActionListener( this );
	m_serverListWindow->setPosition( 50, 40 );
	m_root->add( m_serverListWindow );
	SDL_mutexV( m_mutex );
}

bool WopGUI::handleEvent( SDL_Event* event ) {
	return m_root->handleEvent( event );
}

void WopGUI::draw() {
	SDL_mutexP( m_mutex );
	m_root->draw();
	SDL_mutexV( m_mutex );
}

void WopGUI::createCreditsWindow() {
	WidgetComposite* container = new WidgetComposite();
	VerticalLayout* layout = new VerticalLayout( container );
	container->setLayout( layout );
	
	SDL_Color black = {0, 0, 0, 0};
	TextLabel* label = new TextLabel( "WoP: Wörms of Prey" );
	label->setColor( black );
	container->add( label );
	label = new TextLabel( "(C) 2004-2005 Uwe Fabricius, Thomas Pohl and Christoph Freundl" );
	label->setColor( black );
	container->add( label );
	label = new TextLabel( "WoP comes with ABSOLUTELY NO WARRANTY." );
	label->setColor( black );
	container->add( label );
	label = new TextLabel( "This is free software, and you are welcome to redistribute it" );
	label->setColor( black );
	container->add( label );
	label = new TextLabel( "under certain conditions; see the file LICENSE that comes with" );
	label->setColor( black );
	container->add( label );
	label = new TextLabel( "this distribution." );
	label->setColor( black );
	container->add( label );
	label = new TextLabel( "Intro tune by dK (www.endk.com) - original soundtrack of \"Lost in" );
	label->setColor( black );
	container->add( label );
	label = new TextLabel( "the Underflow\" by Radiant (www.pouet.net/prod.php?which=12246)." );
	label->setColor( black );
	container->add( label );
	label = new TextLabel( "Thanks to Team17 for the permission to use the original Worms(TM) sounds." );
	label->setColor( black );
	container->add( label );
	m_creditsWindow.backButton = new TextButton( "Back" );
	m_creditsWindow.backButton->setHighlightable( true );
	m_creditsWindow.backButton->addActionListener( this );
	m_creditsWindow.backButton->setMinWidth( 540 );
	container->add( m_creditsWindow.backButton );
	
	m_creditsWindow.window = new WopWindow( container, "Credits" );
	m_creditsWindow.window->setPosition
	( ( m_root->getWidth() - m_creditsWindow.window->getWidth() ) / 2,
	  ( m_root->getHeight() - m_creditsWindow.window->getHeight() ) / 2 );
}

void WopGUI::createMainMenu() {
	WidgetComposite* container = new WidgetComposite();
	VerticalLayout* layout = new VerticalLayout( container );
	container->setLayout( layout );

	m_mainMenu.startButton = new TextButton( "Start game" );
	m_mainMenu.startButton->setHighlightable( true );
	m_mainMenu.startButton->addActionListener( this );
	m_mainMenu.startButton->setMinWidth( 300 );
	container->add( m_mainMenu.startButton );
	m_mainMenu.optionsButton = new TextButton( "Options" );
	m_mainMenu.optionsButton->setHighlightable( true );
	m_mainMenu.optionsButton->addActionListener( this );
	m_mainMenu.optionsButton->setMinWidth( 300 );
	container->add( m_mainMenu.optionsButton );
	m_mainMenu.creditButton = new TextButton( "Credits" );
	m_mainMenu.creditButton->setHighlightable( true );
	m_mainMenu.creditButton->addActionListener( this );
	m_mainMenu.creditButton->setMinWidth( 300 );
	container->add( m_mainMenu.creditButton );
	m_mainMenu.exitButton = new TextButton( "Exit" );
	m_mainMenu.exitButton->setHighlightable( true );
	m_mainMenu.exitButton->addActionListener( this );
	m_mainMenu.exitButton->setMinWidth( 300 );
	container->add( m_mainMenu.exitButton );
	
	m_mainMenu.window = new WopWindow( container, "Main menu" );
	m_mainMenu.window->setPosition
	( ( m_root->getWidth() - m_mainMenu.window->getWidth() ) / 2,
	  ( m_root->getHeight() - m_mainMenu.window->getHeight() ) / 2 );
}
